/*
  SRPLUGIN.HPP
  Plugin API for S&R 2.5

  Copyright (c) Ivan Sintyurin (http://www.moris.ru/~spinoza, spinoza@mail.ru)
*/

#ifndef __srplugin_hpp
#define __srplugin_hpp

#if defined(__BORLANDC__)
  #pragma option -a2
#elif defined(__GNUC__) || (defined(__WATCOMC__) && (__WATCOMC__ < 1100)) || defined(__LCC__)
  #pragma pack(2)
  #if defined(__LCC__)
    #define _export __declspec(dllexport)
  #endif
#else
  #pragma pack(push,2)
  #if _MSC_VER
    #define _export
  #endif
#endif

#define MAKE_SANDRVERSION(major,minor,build) (((major)<<8) | (minor) | ((build)<<16))
const DWORD SANDR_VERSION=MAKE_SANDRVERSION(2,5,0);

#define MATCHESNUM 0x10

struct SRPluginInfo
{
  DWORD StructSize;
  DWORD Flags;
  char  MenuString[64];
  DWORD SR_MinVer; //   S&R,    ,
                   //     Run  ( 
                   // SANDR_VERSION)
  DWORD SR_MaxVer; //   S&R,    
                   // ( SR_MaxVer=0,  Run    
                   //  S&R,   SR_MinVer)
};

enum SRPI_FLAGS        //   SRPluginInfo
{
   SRPIF_PANEL  = 0x00000001,
   SRPIF_CONFIG = 0x00000002,
   SRPIF_EDITOR = 0x00000004,
   SRPIF_VIEWER = 0x00000008,
};

const int MAXNAMESIZE=512;

typedef DWORD CSearchRegExp;
typedef DWORD CReplaceRegExp;
typedef DWORD CFileSearch;
typedef DWORD CSchemes;
typedef DWORD EXTENDEDSEARCHOPTIONS;
typedef DWORD CFileMask;
typedef DWORD CMatchHash;
typedef DWORD CFileSecurity;
typedef DWORD CFileAttributes;

struct SMatch
{
  int s,e;
};

//       
struct SRWORKSTATUS
{
  char  file1[MAXNAMESIZE];
  char  file2[MAXNAMESIZE];
  DWORD ExitCode;      // 0  ,   
  DWORD LastError;     //    (GetLastError)
  DWORD Reserved;      //   0
};

enum SRW_ERROR_CODE    //    SRWORKSTATUS.ExitCode
{
  SRWE_OK          =0, //  OK,   
  SRWE_OPENFILE    =1, //               (   .file1)
  SRWE_READFILE    =2, //               (   .file1)
  SRWE_ALLOC       =3, //    
  SRWE_OPENTEMPFILE=4, //     (   .file2)
  SRWE_WRITEFILE   =5, //                 (   .file2)
  SRWE_RENAMEFILE  =6, //              (.file1  .file2)
  SRWE_ESCAPE      =7, //   / Esc/
  SRWE_RRE_FILL    =8, //    
                       // (  CReplaceRegExp)
};

enum FM_FLAGS          //   FM_Set
{
  FMF_SILENT            = 0x00000001,
  FMF_FORBIDEXCLUDE     = 0x00000002,
  FMF_EXPANDENVIRONMENT = 0x00000004,
  FMF_DELETEENDSLASH    = 0x00000008,
  FMF_DONOTSKIPPATH     = 0x00000010,
};

enum FS_SET_FLAGS     //   FS_Set
{
  FSSF_PROCESSSUBDIR  = 0x00000008,
  FSSF_PROCESSDIR     = 0x00000010,
  FSSF_SILENT         = 0x00000020,
  FSSF_ONLYDIR        = 0x00000040,
};

enum FS_SEARCHMODE { FSSM_SELECTION, FSSM_MASK, FSSM_LIST };

enum FS_ERROR_CODE       //   FS_Set
{
  FSE_SUCCESS,           //  
  FSE_FILEMASK,          //   
  FSE_NEEDREALFILEPANEL, //    
  FSE_DIRLIST,           //   
  FSE_ARGUMENT,          //  
  FSE_ALLOC,             //   
  FSE_WRONGSEARCHMODE,   //    
  FSE_EXCLUDEDIRMASK,    //   -
  FSE_ERROR=0xFFFFFFFF,  //  
};

enum CHARACTER_SET_TYPE //  
{
  CST_OEM, CST_ANSI, CST_UNICODE, CST_FAR, CST_ALL
};

//      
struct CHARACTERSET
{
  DWORD CodePage;  // CHARACTER_SET_TYPE
  int   TableNum;  //   
};

enum SRP_OPENFROM {SRPO_PANEL, SRPO_CONFIG, SRPO_EDITOR, SRPO_VIEWER};

//////////////////////////////////////////////////////////////////////////////
enum SR_COMMAND_TYPE{
  SRC_GETVERSION,
  SRC_GETSTRING,
  SRC_GETSETTING,
  SRC_QUESTION,
  SRC_GOTOFILE,
  SRC_ERRORMSG,
  SRC_GETSYSTEMMSG,
  SRC_ASKPROCESSROFILE,
};

enum SR_STRING_ID{
  SRSTR_MASKHIST,
  SRSTR_SEARCHHIST,
  SRSTR_REPLACEHIST,
  SRSTR_DIRECTORYHIST,
  SRSTR_SAVETOFILEHIST,
};

enum SR_SETTING_ID{
  SRSET_NOSLASH,
  SRSET_PSIS,
};

struct SR_CMD_GetString
{
  SR_STRING_ID id;      //  
  char        *Dest;    //      
  DWORD        MaxSize; //   ,    
                        //  (MaxSize-1)
  DWORD        Size;    //    (   )
};

struct SR_CMD_GetSetting
{
  SR_SETTING_ID id;     //  
  DWORD         Value;  //  
};

struct SR_CMD_Question
{
  const char *Title; //  
  const char *Line1; // ,
  const char *Line2; // 
  const char *Line3; //    
  DWORD Flags; // Flags -  ,      FarMessage
};

struct SR_CMD_ErrorMsg
{
  const struct SRWORKSTATUS *srws;
  BOOL   Warning; //   - warning (TRUE)  problem (FALSE)
};

struct SR_CMD_GetSystemMsg
{
  BOOL  UseGetLastError; //  FALSE,      Error,
                         //  -   GetLastError()
  DWORD Error;           //   (,  UseGetLastError=FALSE)
  char *Dest;            //   ,    
                         //  .    
                         //  0x00,    -  
                         // 0x00
  DWORD DestSize;        //  Dest.     3
  DWORD LineSize;        //   .   
                         //       - ,
                         //       
                         //    ,  
                         //   .  LineSize  0,  
                         //    
};

struct SR_CMD_AskProcessROFile
{
  const char *FileName;
  const char *Info;
  const char *Question;
  BOOL  OnlyYesNo;
};

typedef DWORD (WINAPI *SRAPI_CMD)(
  SR_COMMAND_TYPE CMD,
  void *Param
);

//////////////////////////////////////////////////////////////////////////////
typedef int (WINAPI *SRAPI_VIEWERCONTROL)(
  int Command,
  void *Param
);

/////////////////////////////////////////////////////////////////////////////
typedef void (WINAPI *SRAPI_INITLOCMSG)(const char *ModuleName, char *Dest);

typedef char*(WINAPI *SRAPI_LOCMSG)(
  const char *MsgFileName,
  const char *MsgName,
  char *Dest,
  int maxsize
);

//////////////////////////////////////////////////////////////////////////////
// :   
typedef EXTENDEDSEARCHOPTIONS*(WINAPI *SRAPI_ESO_CREATE)(void);
typedef void (WINAPI *SRAPI_ESO_DELETE)(EXTENDEDSEARCHOPTIONS **eso);
typedef BOOL (WINAPI *SRAPI_ESO_GETMODE)(const EXTENDEDSEARCHOPTIONS *eso);

typedef void (WINAPI *SRAPI_ESO_SETMODE)(
  EXTENDEDSEARCHOPTIONS *eso,
  BOOL Mode
);

//    
enum ESO_PARAM_ID{
  ESO_ID_MINSIZE=1,  //   
  ESO_ID_MAXSIZE,    //   
  ESO_ID_PRCMINSIZE, //  ESO_ID_MINSIZE
  ESO_ID_PRCMAXSIZE, //  ESO_ID_MAXSIZE
  ESO_ID_MINTIME,    //       
  ESO_ID_MAXTIME,    //       
  ESO_ID_PRCMINTIME, //  ESO_ID_MINTIME
  ESO_ID_PRCMAXTIME, //  ESO_ID_MACTIME
  ESO_ID_TIMETYPE,   //   (ESO_TIME_TYPE)
  ESO_ID_ATTRIBUTES, //    (AttributesSet)
};

//    (. ESO_PARAM_ID)
enum ESO_TIME_TYPE{
  ESO_TIME_CREATION,
  ESO_TIME_ACCESS,
  ESO_TIME_WRITE,
};

//   (. ESO_PARAM_ID)
struct AttributesSet
{
  BOOL  Process;    //  TRUE.   
  BOOL  SearchInReparsePoint; //   "-"
  DWORD Archive;    //       :
  DWORD Hidden;     // 0 -    
  DWORD ReadOnly;   // 1 -    
  DWORD System;     //    -  
  DWORD Compressed;
  DWORD Encrypted;
};

typedef BOOL (WINAPI *SRAPI_ESO_GETPARAM)(
  const EXTENDEDSEARCHOPTIONS *eso,
  DWORD Param_ID,
  void *Value
);

typedef BOOL (WINAPI *SRAPI_ESO_SETPARAM)(
  EXTENDEDSEARCHOPTIONS *eso,
  DWORD Param_ID,
  void *Value
);

typedef void (WINAPI *SRAPI_ESO_COPY)(
  const EXTENDEDSEARCHOPTIONS *src,
  EXTENDEDSEARCHOPTIONS *dest
);

typedef BOOL (WINAPI *SRAPI_ESO_EDIT)(EXTENDEDSEARCHOPTIONS *eso);

typedef BOOL (WINAPI *SRAPI_ESO_LOAD)(
  const char *RootKey,
  const char *Key,
  EXTENDEDSEARCHOPTIONS *eso
);

typedef BOOL (WINAPI *SRAPI_ESO_SAVE)(
  const char *RootKey,
  const char *Key,
  const EXTENDEDSEARCHOPTIONS *eso
);

typedef BOOL (WINAPI *SRAPI_ESO_COMPAREWFD)(
  const EXTENDEDSEARCHOPTIONS *eso,
  const WIN32_FIND_DATA *wfd
);

//////////////////////////////////////////////////////////////////////////////
//    
typedef CSearchRegExp*(WINAPI *SRAPI_RE_CREATE)(void);
typedef void (WINAPI *SRAPI_RE_DELETE)(CSearchRegExp **re);

typedef BOOL (WINAPI *SRAPI_RE_SETEXPR)(
  CSearchRegExp *re,
  char *Expr,
  struct CharTableSet *CTS,
  DWORD Reserved
);

typedef BOOL (WINAPI *SRAPI_RE_SEARCH)(
  const CSearchRegExp *sre,
  const char *datastart,
  const char *textstart,
  const char *textend,
  struct SMatch *match,
  DWORD *matchcount,
  CMatchHash *hmatch,
  DWORD Reserved
);

typedef BOOL (WINAPI *SRAPI_RE_MATCH)(
  const CSearchRegExp *sre,
  const char *datastart,
  const char *textstart,
  const char *textend,
  struct SMatch *match,
  DWORD *matchcount,
  CMatchHash *hmatch,
  DWORD Reserved
);

typedef int (WINAPI *SRAPI_RE_LASTERROR)(const CSearchRegExp *sre);

//! Possible compile and runtime errors returned by LastError.
enum REError{
  //! No errors
  errNone=0,
  //! RegExp wasn't even tried to compile
  errNotCompiled,
  //! expression contain syntax error
  errSyntax,
  //! Unbalanced brackets
  errBrackets,
  //! Max recursive brackets level reached. Controled in compile time
  errMaxDepth,
  //! Invalid options combination
  errOptions,
  //! Reference to nonexistent bracket
  errInvalidBackRef,
  //! Invalid escape char
  errInvalidEscape,
  //! Invalid range value
  errInvalidRange,
  //! Quantifier applyed to invalid object. f.e. lookahed assertion
  errInvalidQuantifiersCombination,
  //! Size of match array isn't large enough.
  errNotEnoughMatches,
  //! Attempt to match RegExp with Named Brackets, and no storage class provided.
  errNoStorageForNB,
  //! Reference to undefined named bracket
  errReferenceToUndefinedNamedBracket,
  //! Only fixed length look behind assertions are supported
  errVariableLengthLookBehind,
};

typedef int (WINAPI *SRAPI_RE_ERRORPOSITION)(const CSearchRegExp *sre);
typedef int (WINAPI *SRAPI_RE_GETBRACKETSCOUNT)(const CSearchRegExp *sre);

//////////////////////////////////////////////////////////////////////////////
//    
typedef CReplaceRegExp*(WINAPI *SRAPI_RRE_CREATE)(struct SRWORKSTATUS *srws);
typedef void (WINAPI *SRAPI_RRE_DELETE)(CReplaceRegExp **rre);

typedef BOOL (WINAPI *SRAPI_RRE_SETEXPR)(
  CReplaceRegExp *rre,
  const char *Expr,
  const CharTableSet *ctsin,
  const CharTableSet *ctsout,
  DWORD Reserved
);

typedef BOOL (WINAPI *SRAPI_RRE_CHECKLENGTH)(const CReplaceRegExp *rre);

typedef int  (WINAPI *SRAPI_RRE_FILL)(
  CReplaceRegExp *rre,
  SMatch *Match,
  const DWORD TotalMatches,
  const BYTE *fromBuf,
  const WIN32_FIND_DATA *wfd,
  const char *Root,
  CMatchHash *mthsh,
  DWORD Reserved
);

typedef const BYTE*(WINAPI *SRAPI_RRE_GETBUFFER)(const CReplaceRegExp *rre);
typedef void (WINAPI *SRAPI_RRE_SETCOUNTER)(CReplaceRegExp *rre, int value);
typedef int  (WINAPI *SRAPI_RRE_GETCOUNTER)(const CReplaceRegExp *rre);

//////////////////////////////////////////////////////////////////////////////
//      
typedef void (WINAPI *FSPROCESSITEMCALLBACK)(
  WIN32_FIND_DATA *wfd,
  void *UserData
);

typedef void (WINAPI *FSPROCESSCHANGEDIRCALLBACK)(
  void *UserData,
  DWORD *Param,
  int Reason
);

typedef void (WINAPI *SHOWPROGRESS)(const char *Name);

typedef CFileSearch*(WINAPI *SRAPI_FS_CREATE)(void);
typedef void (WINAPI *SRAPI_FS_DELETE)(CFileSearch **fs);

typedef DWORD (WINAPI *SRAPI_FS_SET)(
  CFileSearch *fs,
  DWORD searchmode,
  DWORD Flags,
  const EXTENDEDSEARCHOPTIONS *eso,
  const char *root,
  const char *mask,
  const struct PanelInfo *panelInfo,
  struct SRWORKSTATUS *srws,
  DWORD Reserved
);

typedef void (WINAPI *SRAPI_FS_SEARCH)(
  CFileSearch *fs,
  void *UserData,
  FSPROCESSCHANGEDIRCALLBACK prcCDcb,
  SHOWPROGRESS showprogress,
  FSPROCESSITEMCALLBACK pf,
  DWORD Reserved
);

typedef const char * (WINAPI *SRAPI_FS_GETROOT)(const CFileSearch *fs);

//////////////////////////////////////////////////////////////////////////////
//   ""
typedef BOOL (WINAPI *LOADSCHEME)(void *dest, const char *key);
typedef BOOL (WINAPI *SAVESCHEME)(const void *src, const char *key);
typedef int (WINAPI *EDITSCHEME)(void *data,  const char *Name);

typedef CSchemes*(WINAPI *SRAPI_SCHEME_CREATE)(
  const char *RootKey,
  const char *Key,
  const char *Name,
  const char *Total,
  const char *Title,
  void *original,
  void *copy,
  LOADSCHEME loadscheme,
  SAVESCHEME savescheme,
  EDITSCHEME EditScheme,
  DWORD Reserved
);

typedef void (WINAPI *SRAPI_SCHEME_DELETE)(CSchemes **scheme);
typedef int  (WINAPI *SRAPI_SCHEME_MENU)(CSchemes *scheme, int DefaultPos);
typedef BOOL (WINAPI *SRAPI_SCHEME_ADD)(CSchemes *scheme, const void *src);

//////////////////////////////////////////////////////////////////////////////
//   
typedef BOOL (WINAPI *SRAPI_WRITESTR)(
  const char *RootKey,
  const char *Key,
  const char *ValueName,
  const char *ValueData
);

typedef int  (WINAPI *SRAPI_READSTR)(
  const char *RootKey,
  const char *Key,
  const char *ValueName,
  char *ValueData,
  const char *Default,
  DWORD DataSize
);

typedef BOOL (WINAPI *SRAPI_WRITEDWORD)(
  const char *RootKey,
  const char *Key,
  const char *ValueName,
  DWORD ValueData
);

typedef int  (WINAPI *SRAPI_READDWORD)(
  const char *RootKey,
  const char *Key,
  const char *ValueName,
  DWORD Default
);

typedef BOOL (WINAPI *SRAPI_WRITEBYTES)(
  const char *RootKey,
  const char *Key,
  const char *ValueName,
  const BYTE *ValueData,
  DWORD ValueSize
);

typedef int  (WINAPI *SRAPI_READBYTES)(
  const char *RootKey,
  const char *Key,
  const char *ValueName,
  BYTE *ValueData,
  const BYTE *Default,
  DWORD DataSize
);

typedef BOOL (WINAPI *SRAPI_DELETEKEY)(
  const char *RootKey,
  const char *Key
);

//////////////////////////////////////////////////////////////////////////////
//    
typedef CFileMask*(WINAPI *SRAPI_FM_CREATE)(void);

typedef void (WINAPI *SRAPI_FM_DELETE)(
  CFileMask **fm
);

typedef BOOL (WINAPI *SRAPI_FM_SET)(
  CFileMask *fm,
  const char *Masks,
  DWORD Flags
);

typedef BOOL (WINAPI *SRAPI_FM_COMPARE)(
  const CFileMask *fm,
  const char *Name
);


//////////////////////////////////////////////////////////////////////////////
//    
typedef CFileSecurity *(WINAPI *SRAPI_CFSEC_CREATE)(void);

typedef void (WINAPI *SRAPI_CFSEC_DELETE)(
  CFileSecurity **fsec
);

typedef BOOL (WINAPI *SRAPI_CFSEC_GETSECURITY)(
  CFileSecurity *fsec,
  const char *FileName
);

typedef BOOL (WINAPI *SRAPI_CFSEC_SETSECURITY)(
  CFileSecurity *fsec,
  const char *FileName
);

//////////////////////////////////////////////////////////////////////////////
//    
typedef CFileAttributes *(WINAPI *SRAPI_CFATTR_CREATE)(void);

typedef void (WINAPI *SRAPI_CFATTR_DELETE)(
  CFileAttributes **fattr
);

typedef DWORD (WINAPI *SRAPI_CFATTR_GETATTR)(
  CFileAttributes *fattr,
  const char *FileName
);

typedef BOOL (WINAPI *SRAPI_CFATTR_SETATTR)(
  CFileAttributes *fattr,
  const char *FileName,
  DWORD NewAttr
);

//////////////////////////////////////////////////////////////////////////////
//  
enum CHARACTERSET_FLAGS { CSF_ENABLEUNICODE=0x0001, CSF_ENABLEALL=0x0002 };

typedef BOOL (WINAPI *SRAPI_CS_LOAD)(
  const struct CHARACTERSET *CS,
  struct CharTableSet *CTS
);

typedef BOOL (WINAPI *SRAPI_CS_CHOOSE)(
  struct CHARACTERSET *CS,
  DWORD  Flags
);

//////////////////////////////////////////////////////////////////////////////
//   CMatchHash
typedef CMatchHash*(WINAPI *SRAPI_CHASH_CREATE)();
typedef void (WINAPI *SRAPI_CHASH_DELETE)(CMatchHash **chash);

typedef int (WINAPI *SRAPI_CHASH_EXISTS)(
  CMatchHash *chash,
  const char *Key
);

typedef SMatch *(WINAPI *SRAPI_CHASH_SETITEM)(
  CMatchHash *chash,
  const char *Key,
  const SMatch *Value
);

typedef SMatch *(WINAPI *SRAPI_CHASH_GETITEMADDR)(
  CMatchHash *chash,
  const char*Key
);

typedef void (WINAPI *SRAPI_CHASH_FIRST)(CMatchHash *chash);
typedef SMatch *(WINAPI *SRAPI_CHASH_GETNEXT)(CMatchHash *chash);
typedef void (WINAPI *SRAPI_CHASH_EMPTY)(CMatchHash *chash);

//////////////////////////////////////////////////////////////////////////////
struct SRPluginStartupInfo
{
  DWORD                       StructSize;
  SRAPI_CMD                   Cmd;
  SRAPI_INITLOCMSG            InitLocMsg;
  SRAPI_LOCMSG                LocMsg;
  DWORD                       Reserved1[2];

//////////////////////////////////////////////////////////////////////////////
// Viewer API
  SRAPI_VIEWERCONTROL         ViewerControl;
  DWORD                       Reserved2[2];

//////////////////////////////////////////////////////////////////////////////
// :   
  SRAPI_ESO_CREATE            ESO_Create;
  SRAPI_ESO_DELETE            ESO_Delete;
  SRAPI_ESO_GETMODE           ESO_GetMode;
  SRAPI_ESO_SETMODE           ESO_SetMode;
  SRAPI_ESO_GETPARAM          ESO_GetParam;
  SRAPI_ESO_SETPARAM          ESO_SetParam;
  SRAPI_ESO_COPY              ESO_Copy;
  SRAPI_ESO_EDIT              ESO_Edit;
  SRAPI_ESO_LOAD              ESO_Load;
  SRAPI_ESO_SAVE              ESO_Save;
  SRAPI_ESO_COMPAREWFD        ESO_CompareWithWFD;
  DWORD                       Reserved3[1];

//////////////////////////////////////////////////////////////////////////////
//    
  SRAPI_RE_CREATE             RE_Create;
  SRAPI_RE_DELETE             RE_Delete;
  SRAPI_RE_SETEXPR            RE_SetExpr;
  SRAPI_RE_SEARCH             RE_Search;
  SRAPI_RE_MATCH              RE_Match;
  SRAPI_RE_LASTERROR          RE_LastError;
  SRAPI_RE_ERRORPOSITION      RE_ErrorPosition;
  SRAPI_RE_GETBRACKETSCOUNT   RE_GetBracketsCount;
  DWORD                       Reserved4[2];

//////////////////////////////////////////////////////////////////////////////
//    
  SRAPI_RRE_CREATE            RRE_Create;
  SRAPI_RRE_DELETE            RRE_Delete;
  SRAPI_RRE_SETEXPR           RRE_SetExpr;
  SRAPI_RRE_CHECKLENGTH       RRE_CheckLength;
  SRAPI_RRE_FILL              RRE_Fill;
  SRAPI_RRE_GETBUFFER         RRE_GetBuffer;
  SRAPI_RRE_SETCOUNTER        RRE_SetCounter;
  SRAPI_RRE_GETCOUNTER        RRE_GetCounter;
  DWORD                       Reserved5[2];

//////////////////////////////////////////////////////////////////////////////
//      
  SRAPI_FS_CREATE             FS_Create;
  SRAPI_FS_DELETE             FS_Delete;
  SRAPI_FS_SET                FS_Set;
  SRAPI_FS_SEARCH             FS_Search;
  SRAPI_FS_GETROOT            FS_GetRoot;
  DWORD                       Reserved6[2];

//////////////////////////////////////////////////////////////////////////////
//   ""
  SRAPI_SCHEME_CREATE         SCHEME_Create;
  SRAPI_SCHEME_DELETE         SCHEME_Delete;
  SRAPI_SCHEME_MENU           SCHEME_Menu;
  SRAPI_SCHEME_ADD            SCHEME_Add;
  DWORD                       Reserved7[2];

//////////////////////////////////////////////////////////////////////////////
//   
  SRAPI_WRITESTR              WriteStr;
  SRAPI_READSTR               ReadStr;
  SRAPI_WRITEDWORD            WriteDWORD;
  SRAPI_READDWORD             ReadDWORD;
  SRAPI_WRITEBYTES            WriteBytes;
  SRAPI_READBYTES             ReadBytes;
  SRAPI_DELETEKEY             DeleteKey;
  DWORD                       Reserved8[2];

//////////////////////////////////////////////////////////////////////////////
//    
  SRAPI_FM_CREATE             FM_Create;
  SRAPI_FM_DELETE             FM_Delete;
  SRAPI_FM_SET                FM_Set;
  SRAPI_FM_COMPARE            FM_Compare;
  DWORD                       Reserved9[2];

//////////////////////////////////////////////////////////////////////////////
//  
  SRAPI_CS_LOAD               CS_Load;
  SRAPI_CS_CHOOSE             CS_Choose;
  DWORD                       Reserved10[2];

//////////////////////////////////////////////////////////////////////////////
//   CMatchHash
  SRAPI_CHASH_CREATE          Hash_Create;
  SRAPI_CHASH_DELETE          Hash_Delete;
  SRAPI_CHASH_EXISTS          Hash_Exists;
  SRAPI_CHASH_SETITEM         Hash_SetItem;
  SRAPI_CHASH_GETITEMADDR     Hash_GetItemAddr;
  SRAPI_CHASH_FIRST           Hash_First;
  SRAPI_CHASH_GETNEXT         Hash_GetNext;
  SRAPI_CHASH_EMPTY           Hash_Empty;
  DWORD                       Reserved11[2];

//////////////////////////////////////////////////////////////////////////////
//    
  SRAPI_CFSEC_CREATE          FSec_Create;
  SRAPI_CFSEC_DELETE          FSec_Delete;
  SRAPI_CFSEC_GETSECURITY     FSec_GetSecurity;
  SRAPI_CFSEC_SETSECURITY     FSec_SetSecurity;
  DWORD                       Reserved12[2];

//////////////////////////////////////////////////////////////////////////////
//    
  SRAPI_CFATTR_CREATE         FAttr_Create;
  SRAPI_CFATTR_DELETE         FAttr_Delete;
  SRAPI_CFATTR_GETATTR        FAttr_GetAttr;
  SRAPI_CFATTR_SETATTR        FAttr_SetAttr;
  DWORD                       Reserved13[2];

  DWORD                       Reserved14[10];
};

#if defined(__BORLANDC__) || defined(_MSC_VER) || defined(__GNUC__) || defined(__WATCOMC__)
#ifdef __cplusplus
extern "C"{
#endif
// Exported Functions

extern "C"
{
  void WINAPI _export GetInfo (struct SRPluginInfo *SRPInfo);
  void WINAPI _export Start (const struct PluginStartupInfo *Info,
                             const struct SRPluginStartupInfo *srInfo);
  void WINAPI _export Exit ();
  int WINAPI _export Run (DWORD OpenFrom, DWORD Reserved1, DWORD Reserved2);
}

#ifdef __cplusplus
};
#endif
#endif

#if defined(__BORLANDC__)
  #pragma option -a.
#elif defined(__GNUC__) || (defined(__WATCOMC__) && (__WATCOMC__ < 1100)) || defined(__LCC__)
  #pragma pack()
#else
  #pragma pack(pop)
#endif

#endif // __srplugin_hpp
